#include "radian.h"
Radian::Radian(short _a = 0, short _b = 0, short _c = 0, short _d = 0) {
	long a, b, c, d;
	long i, j;

	j = _c * 100 + _d;

	i = j / 6000;
	d = j % 6000;
	c = (d * 10 ) / 6;

	j = _b + i;
	i = j / 60;
	b = j % 60;

	j = _a + i;
	a = j % 360;
	if (a > 180) {
		a = a - 360;
	}
	_data = a * 60 * 10000 + b * 10000 + c;
}

long Radian::toLong() {
	return _data;
}

void TokyoToWGS84(Radian * radNS, Radian * radEW) {
	long newNS, newEW;
	newNS = radNS->toLong() - 0.00010695 * (radNS->toLong()) - 0.000017464
			* (radEW->toLong()) - 2761;
	newEW = radEW->toLong() + 0.000046038 * (radNS->toLong()) - 0.000083043
			* (radEW->toLong()) + 6025;
	radNS->setData(newNS);
	radEW->setData(newEW);
}

void WGS84ToTokyo(Radian * radNS, Radian * radEW) {
	long newNS, newEW;
	newNS = radNS->toLong() + 0.00010696 * (radNS->toLong()) + 0.000017467
			* (radEW->toLong()) + 2761;
	newEW = radEW->toLong() - 0.000046047 * (radNS->toLong()) + 0.000083049
			* (radEW->toLong()) - 6025;
	radNS->setData(newNS);
	radEW->setData(newEW);
}

void Radian::setData(long data) {
	_data = data;
}

bool Radian::parse(const QString &str) {
	long c, data;

	// ٿѴ
	QStringList rads = QStringList::split(".", str.mid(1));
	if (rads.count() == 2) {
		c = rads[1].toInt();
		data = (rads[0].toInt() / 100) * 600000 + (rads[0].toInt() % 100)
				* 10000 + c;
	} else if (rads.count() == 4) {
		c = (rads[2].toInt() * 100 + rads[3].toInt()) * 10 / 6;
		data = rads[0].toInt() * 600000 + rads[1].toInt() * 10000 + c;
	} else {
		return false;
	}

	// +-Ƚ
	if (str.startsWith("S") || str.startsWith("E")) {
		_data = data;
	} else if (str.startsWith("N") || str.startsWith("W")) {
		_data = data * -1;
	} else {
		return false;
	}

	return true;
}

bool Radian::parse2(const QString &str) {
	long a, b, data;
	double base;

	// ٿѴ
	QStringList rads = QStringList::split(".", str.mid(1));
	if (rads.count() == 2) {
		base = str.mid(1).toDouble();
		a = (long)base;
		base = (base - a) * 600000;
		b = (long)base;
		data = a * 600000 + b;
	} else {
		return false;
	}

	// +-Ƚ
	if (str.startsWith("S") || str.startsWith("E")) {
		_data = data;
	} else if (str.startsWith("N") || str.startsWith("W")) {
		_data = data * -1;
	} else {
		return false;
	}

	return true;
}

short Radian::getMin() {
	return (((_data % 10000) * 6) + 5) / 10;
}

void Radian::setMin(short _c, short _d = 0) {
	long i, j;

	i = _data - (_data % 10000);
	j = _c * 100 + _d;
	_data = i + (j * 10 ) / 6;
}

QString Radian::toString() {
	QString rtn, temp;
	int c;
	long data;

	if (_data >= 0) {
		data = _data;
	} else {
		data = _data * -1;
		rtn = "-";
	}

	rtn += QString::number(data / 600000);
	rtn += ".";
	rtn += QString::number((data % 600000) / 10000);
	rtn += ".";
	c = ((data % 10000) * 6 + 5) / 10;
	rtn += QString::number(c / 100);
	rtn += ".";
	rtn += ("0" + QString::number(c % 100)).right(2);
	return rtn;
}

QString Radian::toString2(bool multiplyMinus = false) {
	int x = 1;
	if (multiplyMinus)
		x = -1;

	return QString::number(_data * x / 600000.0, 'f', 8);
}

QString Radian::toNS() {
	QString rtn, temp;
	int c;
	long data;

	if (_data >= 0) {
		data = _data;
		rtn = "S";
	} else {
		data = _data * -1;
		rtn = "N";
	}

	rtn += QString::number(data / 600000);
	rtn += ".";
	rtn += QString::number((data % 600000) / 10000);
	rtn += ".";
	c = ((data % 10000) * 6 + 5) / 10;
	rtn += QString::number(c / 100);
	rtn += ".";
	rtn += ("0" + QString::number(c % 100)).right(2);
	return rtn;
}

QString Radian::toEW() {
	QString rtn, temp;
	int c;
	long data;

	if (_data >= 0) {
		data = _data;
		rtn = "E";
	} else {
		data = _data * -1;
		rtn = "W";
	}

	rtn += QString::number(data / 600000);
	rtn += ".";
	rtn += QString::number((data % 600000) / 10000);
	rtn += ".";
	c = ((data % 10000) * 6 + 5) / 10;
	rtn += QString::number(c / 100);
	rtn += ".";
	rtn += ("0" + QString::number(c % 100)).right(2);
	return rtn;
}

Radian& Radian::operator=(Radian value) {
	_data = value.toLong();
	return *this;
}
