#ifndef ZEGA_H
#define ZEGA_H
#include<qwidget.h>
#include<qpe/config.h>
#include<qmenubar.h>
#include<qpopupmenu.h>
#include<qapplication.h>
#include<qmessagebox.h>
#include"qtkfiledialog.h"
#include<qimage.h>
#include<qpainter.h>
#include<qtimer.h>
#include"maplist.h"
#include"custommaplist.h"
#include"mobilemapnavimaps.h"
#include"gpsloghandler.h"
#include"mapCache.h"

/*
 #define _DEBUG_PRINT
*/
#define PIXEL_BASE 4

class Zega : public QWidget {
	Q_OBJECT

public:
	Zega(QWidget* parent, const char* name);
	~Zega();

	// static member
	static Config config;

protected:
	void paintEvent(QPaintEvent*);

private:
	void mousePressEvent(QMouseEvent *event);
	void resizeEvent(QResizeEvent *event);
	void keyPressEvent(QKeyEvent *event);
	void paintMaps(Radian radNS, Radian radEW);
	void drawRoute();
	void drawGpsLog();
	void drawDestinationLine();
	double getLength(Coordinate &aPoint, Coordinate &bPoint);
	void getEdgePoint(int *x, int *y, long dEW, long dNS);
	bool copyMap(int x, int y, Radian radEW, Radian radNS);
	void disableSuspend();
	void enableSuspend();

	QMenuBar *_menu;
	QPopupMenu *_filemenu;
	QPopupMenu *_gpsmenu;
	QPopupMenu *_scalemenu;
	QPopupMenu *_editmenu;
	QPopupMenu *_displaymenu;
	QPopupMenu *_helpmenu;
	MapCache *_mapCache;
	QImage *_buffer;
	QPixmap *_mapBuffer;
	MapList *_mapList;
	MobileMapNaviMaps *_mobileMapNaviMaps;
	CustomMapList _customMapList;
	int _mapIndex;
	int _autoCenteredBounds;
	GpsLogHandler *_gpsLogHandler;
	QStringList mapDirs;

	// ̹ѤΥޡ
	QTimer *_updateTimer;

	Radian _centerNS;
	Radian _centerEW;
	long _pixelNS;
	long _pixelEW;
	int  _pixelBase;
	int _bsl, _bsr, _bsu, _bsd;
	bool _autoCentered;
	bool _showGpsLog;
	bool _showRoute;
	bool _showDestinationLine;
	bool _evenRePaint;
	
	// ϩ
	vector<Coordinate> _route;
	double _routeLength;
	
	// 
	QColor _routeColor;
	QColor _recentLogColor;
	QColor _obsoluteLogColor;
	QColor _currentPointColor;
	QColor _destinationLineColor;
	
	// 
	int _routeWidth;
	int _logWidth;
	int _destinationLineWidth;
	
	// ư
	int _cursorMoveDen;
	int _shiftCursorMoveDen;
	
private slots:
	void about();
void slotLoad();
void slotSaveRoute();
void slotChScale(int);
void slotRePaint();
void startGps();
void stopGps();
void slotToggleShowGpsLog();
void slotToggleShowRoute();
void slotToggleShowDestinationLine();
void slotToggleAutoCentered();
void slotAddRoutePoint();
void slotDelRoutePoint();
void slotReverseRoute();
void slotClearRoute();

};

#endif //ZEGA_H
